package hpdos_rdma_offloaded;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.util.concurrent.ForkJoinPool;

import hpdos_rdma_offloaded.lib.Property;

import com.ibm.darpc.DaRPCServerEndpoint;
import com.ibm.darpc.DaRPCServerGroup;
import com.ibm.disni.RdmaServerEndpoint;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;
import org.rocksdb.RocksDB;

import hpdos_rdma_offloaded.handler.NetworkHandlerM;
import hpdos_rdma_offloaded.handler.StorageHandler;
import hpdos_rdma_offloaded.protocol.Request;
import hpdos_rdma_offloaded.protocol.Response;
import hpdos_rdma_offloaded.protocol.RpcProtocol;
import hpdos_rdma_offloaded.service.FollowerMetadataService;
import hpdos_rdma_offloaded.service.MetadataServiceMaster;
import rdma.DaRPCClientGroupM;

public class MetadataServer{
    
    StorageHandler storageHandler;
    
    MetadataServiceMaster metadataService;
    FollowerMetadataService followerMetadataService;

    final static Logger logger = Logger.getLogger(MetadataServer.class);    

    public void run(Property property){
        try
        {
            DaRPCServerGroup<Request, Response> group = null;
            if(property.isMaster)
            { 
                logger.debug("STARTING MASTER SERVICES.");
                NetworkHandlerM networkHandler =  new NetworkHandlerM();
                RpcProtocol rpcProtocol = new RpcProtocol();
                DaRPCClientGroupM<Request,Response> followerClientGroup =  DaRPCClientGroupM.createClientGroup(rpcProtocol, 100, 10, 16, 16);
                for(int i=0;i<property.nFollowers;i++)
                {
                    var endpoint =  followerClientGroup.createEndpoint();
                    endpoint.connect(new InetSocketAddress(property.followersIP[i], property.port), 100);
                    if(endpoint.isConnected())
                       networkHandler.addReplicationStream(endpoint.createStream());
                }
                var metadataServerService =  new MetadataServiceMaster(networkHandler,storageHandler);
                group = DaRPCServerGroup.createServerGroup(metadataServerService, property.clusterAffinities,
                                    1000, property.maxinline, property.polling, property.recvQueue,
                                    property.sendQueue, property.wqSize, property.clusterSize);        
                ForkJoinPool.commonPool().execute(networkHandler);
            }
            else
            {
                logger.debug("STARTING FOLLOWER SERVICES.");
                //registrationHandler = new RegistrationHandler(isMaster, host, masterIp, registrationPort, networkHandler);
                var followerMetadataServerService =  new FollowerMetadataService(storageHandler);
                group = DaRPCServerGroup.createServerGroup(followerMetadataServerService, property.clusterAffinities,
                                1000, property.maxinline, property.polling, property.recvQueue,
                                property.sendQueue, property.wqSize, property.clusterSize);
            }

            RdmaServerEndpoint<DaRPCServerEndpoint<Request, Response>> serverEp = group.createServerEndpoint();
            InetSocketAddress address = new InetSocketAddress(property.host, property.port);
            serverEp.bind(address, 100);

            while(true){
                try
                {
                logger.debug("Listening to RDMA requests a, IP: " + property.host + " , PORT: " + property.port);
                    serverEp.accept();
                logger.debug("Accepted connection.");
                }
                catch (IOException e)
                {
                    e.printStackTrace();
                }
            }
        }catch(Exception e)
        {
            e.printStackTrace();
        }
    }

    public static void main(String[] args)  throws Exception
    {
        Property property = new Property(args[0]);
        RocksDB.loadLibrary();
        BasicConfigurator.configure();
        var server = new MetadataServer();
        server.storageHandler = new StorageHandler(property.dbPath);
        server.run(property);
    }
}
