package hpdos_rdma_offloaded.handler;

import java.io.IOException;
import java.util.concurrent.CopyOnWriteArrayList;

import com.ibm.disni.util.NativeAffinity;

import hpdos_rdma_offloaded.protocol.InvalidationRequest;
import hpdos_rdma_offloaded.protocol.InvalidationResponse;
import hpdos_rdma_offloaded.protocol.Request;
import hpdos_rdma_offloaded.protocol.RequestType;
import hpdos_rdma_offloaded.protocol.Response;
import rdma.DaRPCFutureM;
import rdma.DaRPCStreamM;

public class NetworkHandlerM implements Runnable{

    private CopyOnWriteArrayList<DaRPCStreamM<InvalidationRequest, InvalidationResponse>> invalidationStreams;
    private CopyOnWriteArrayList<DaRPCStreamM<Request, Response>> replicationStreams;

    public NetworkHandlerM()
    {
        this.invalidationStreams = new CopyOnWriteArrayList<>();
        this.replicationStreams = new CopyOnWriteArrayList<>();
    }
    public void addInvalidationStream(DaRPCStreamM<InvalidationRequest, InvalidationResponse> stream){
        this.invalidationStreams.add(stream);
    }
    public void addReplicationStream(DaRPCStreamM<Request,Response> stream)
    {
        this.replicationStreams.add(stream);
    }

    public void run()
    {
        NativeAffinity.setAffinity(1<<1);
        while(true)
        {
        for(var stream : invalidationStreams)
        {
            try
            {
                DaRPCFutureM<InvalidationRequest,InvalidationResponse> future = stream.processStream();
                if(future.metadataResponse.invalidationTotal == future.metadataResponse.invalidationDone.incrementAndGet())
                {
                    if(Response.DONE == future.metadataResponse.state.incrementAndGet())
                        future.metadataResponse.event.triggerResponse();
                    
                }
            }catch(Exception exx)
            {
                exx.printStackTrace();
            }
        }
        for(var stream : replicationStreams)
        {
            try
            {
                DaRPCFutureM<Request,Response> future = stream.processStream();
                if(future.metadataResponse.replicatonTotal == future.metadataResponse.replicationDone.incrementAndGet())
                   {
                    if(Response.DONE == future.metadataResponse.state.incrementAndGet())
                        future.metadataResponse.event.triggerResponse();
                    
                }
            }catch(Exception exx)
            {
                exx.printStackTrace();
            }
        }
        }
    }
    public void sendInvalidations(Request request,Response response)
    {
        InvalidationRequest inRequest =  new InvalidationRequest();

        inRequest.setKey( request.getKey());
        response.invalidationTotal = invalidationStreams.size();

        for(var stream : invalidationStreams)
        {
            try
            {
                stream.request(inRequest, new InvalidationResponse(), response, false);
            }
            catch(IOException ex)
            {
                System.out.println(ex);
            }
        }   
    }


    public void sendReplications(Request request,Response response)
    {
        Request repRequest =  new Request();
        repRequest.setRequestType(request.getRequestType());
        repRequest.setKey(request.getKey());

        if(request.getRequestType() == RequestType.PUT)
        {
            repRequest.setValue(request.getValue());
        }
        
        response.replicatonTotal = replicationStreams.size();

        for(var stream : replicationStreams)
        {
            try
            {
                stream.request(repRequest, new Response(), response, false);
            }
            catch(IOException ex)
            {
                System.out.println(ex);
            }
        }   
    }
}
