package hpdos_rdma_offloaded.handler;

import org.rocksdb.RocksDB;
import org.apache.log4j.Logger;
import org.rocksdb.Options;
import org.rocksdb.RocksDBException;

import hpdos_rdma_offloaded.protocol.Request;


public class StorageHandler implements AutoCloseable{

    RocksDB rocksDB;
    Options rockDbOptions;
    final static Logger logger = Logger.getLogger(StorageHandler.class);
    public StorageHandler(String dbpath) throws RocksDBException
    {
        logger.debug("Creating RocksDB");
        this.rockDbOptions = new Options();
        rockDbOptions.setCreateIfMissing(true);
        this.rocksDB = RocksDB.open(rockDbOptions,dbpath);
        logger.debug("Created RocksDB");
    }
    public void close()
    {
        rocksDB.close();
        rockDbOptions.close();
        logger.debug("Closing RocksDB instance");
    }

    public void create(byte[] key,byte[] value) throws RocksDBException{
        rocksDB.put(key,value);
    }
    public void create(Request request) throws RocksDBException {
        logger.debug("Writing to local");
        rocksDB.put(request.getKey(),request.getValue());
    }
    public byte[] read(byte[] key)throws RocksDBException
    {
        return rocksDB.get(key);
    }
    public byte[] read(Request request)throws RocksDBException
    {
        return rocksDB.get(request.getKey());
    }
    public void update(byte[] key,byte[] value) throws RocksDBException
    {
        rocksDB.put(key,value);
    }
    public void update(Request request) throws RocksDBException
    {
        rocksDB.put(request.getKey(),request.getValue());
    }
    public void delete(byte[] key) throws RocksDBException
    {
        rocksDB.delete(key);
    }
    public void delete(Request request) throws RocksDBException
    {
        rocksDB.delete(request.getKey());
    }
}