package hpdos_rdma_offloaded.service;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.util.concurrent.ForkJoinPool;
import java.util.concurrent.TimeUnit;

import com.google.common.base.Stopwatch;

import com.ibm.darpc.DaRPCServerEndpoint;
import com.ibm.darpc.DaRPCServerEvent;
import com.ibm.darpc.DaRPCService;

import org.apache.log4j.Logger;
import org.rocksdb.RocksDBException;

//import hpdos_rdma_offloaded.ExperimentStatistics;
import hpdos_rdma_offloaded.handler.NetworkHandlerM;
import hpdos_rdma_offloaded.handler.StorageHandler;
import hpdos_rdma_offloaded.protocol.AckType;
import hpdos_rdma_offloaded.protocol.InvalidationRequest;
import hpdos_rdma_offloaded.protocol.InvalidationResponse;
import hpdos_rdma_offloaded.protocol.InvalidationRpcProtocol;
import hpdos_rdma_offloaded.protocol.Request;
import hpdos_rdma_offloaded.protocol.RequestType;
import hpdos_rdma_offloaded.protocol.Response;
import hpdos_rdma_offloaded.protocol.RpcProtocol;

import rdma.DaRPCClientGroupM;

public class MetadataServiceMaster extends RpcProtocol implements DaRPCService<Request, Response>{
    NetworkHandlerM networkHandler;
    StorageHandler storageHandler;
    DaRPCClientGroupM<InvalidationRequest, InvalidationResponse> invalidationClientGroup;

    final static Logger logger = Logger.getLogger(MasterMetadataServerService.class);

    public MetadataServiceMaster(NetworkHandlerM networkHandler,StorageHandler storageHandler) throws Exception{
        logger.debug("New MetadataServerService Created");
        this.networkHandler = networkHandler;
        this.storageHandler  = storageHandler;

        InvalidationRpcProtocol rpcProtocol = new InvalidationRpcProtocol();
        invalidationClientGroup  =  DaRPCClientGroupM.createClientGroup(rpcProtocol, 100, 10, 16, 16);  
    }

    @Override
    public void processServerEvent(DaRPCServerEvent<Request, Response> event) throws IOException {
        Request request = event.getReceiveMessage();
        Response response = event.getSendMessage();
        logger.info("Received "+request.getRequestType()+" Request for key: " + new String(request.getKey()));
        Stopwatch stopwatch = Stopwatch.createUnstarted();
        long startTime, endTime, timeTaken;
        if(request.getValue() != null)
            logger.info(" value : "+ new String(request.getValue()));
        try{
            if (request.getRequestType() == RequestType.PUT) {
                startTime = System.nanoTime();
                this.storageHandler.create(request);
                endTime = System.nanoTime();
                timeTaken = (endTime-startTime) / 1000;

                response.event = event;
                this.networkHandler.sendInvalidations(request, response);
                this.networkHandler.sendReplications(request, response);
 //               ExperimentStatistics.collectStatistics(timeTaken, "localWriteTime");
                logger.info("Time to write to local cache: " + timeTaken);

                response.setAck(AckType.SUCCESS);
                response.setKey(null);
                response.setValue(null);
            } 
            else if(request.getRequestType() == RequestType.GET) {
                stopwatch.start();
                byte[] value = this.storageHandler.read(request); 
                stopwatch.stop();
                logger.info("Time to read: " + stopwatch.elapsed(TimeUnit.MICROSECONDS));
                stopwatch.reset();
                if(value == null)
                {
                    response.setAck(AckType.NOTFOUND);
                }
                else
                {
                    response.setAck(AckType.SUCCESS_WITH_VALUE);
                    response.setValue(value);
                }
                event.triggerResponse();
            }
            else if(request.getRequestType() == RequestType.DELETE){
                stopwatch.start();
                this.storageHandler.delete(request);
                stopwatch.stop();
                logger.info("Time to delete to local cache: " + stopwatch.elapsed(TimeUnit.MICROSECONDS));
                stopwatch.reset();

                response.event = event;
                this.networkHandler.sendInvalidations(request, response);
                this.networkHandler.sendReplications(request, response);
                response.setAck(AckType.SUCCESS);
            }
            else{
                response.setAck(AckType.INVALID_REQUEST);
                event.triggerResponse();
            }
        }
        catch(RocksDBException ex){
                ex.printStackTrace();
                response.setAck(AckType.DBFAILED);
                event.triggerResponse();
        }
    }

    @Override
    public void open(DaRPCServerEndpoint<Request, Response> rpcClientEndpoint) {
        logger.debug("Received new connection  ");

        ForkJoinPool.commonPool().submit(()->{
            try{
                System.out.println(rpcClientEndpoint.getDstAddr()+" "+rpcClientEndpoint.getSrcAddr());
                String clientIP = ((InetSocketAddress)rpcClientEndpoint.getDstAddr()).getHostName();
                InetSocketAddress clientSocketAddress = new InetSocketAddress(clientIP,1921);
                System.out.println("Creating Endpoint");
                var invalidationClientEp = invalidationClientGroup.createEndpoint();
                invalidationClientEp.connect(clientSocketAddress, 100);
                System.out.println("accepted");
                if(invalidationClientEp.isConnected()){
                    this.networkHandler.addInvalidationStream(invalidationClientEp.createStream());
                }
                else{
                    System.out.println("Not able to Connect to client for invalidation");
                }
            }
            catch(Exception e){
                e.printStackTrace();
            }
        });
    }

    @Override
    public void close(DaRPCServerEndpoint<Request, Response> rpcClientEndpoint) {
        try
        {
            System.out.println("closed"+((InetSocketAddress)rpcClientEndpoint.getDstAddr()).getHostName());
            
        } catch (IOException e) {
            e.printStackTrace();
        }
        
    }
    
}
